<?php

declare(strict_types=1);

namespace App\Service;

use App\Entity\Order;
use App\Entity\OrderItem;
use App\Enum\OrderStatus;
use App\Repository\DrinkTypeRepository;
use App\Repository\OrderItemRepository;
use App\Repository\OrderRepository;
use DateTimeImmutable;
use InvalidArgumentException;

readonly class OrderService
{
    public function __construct(
        private OrderRepository $orderRepository,
        private OrderItemRepository $orderItemRepository,
        private DrinkTypeRepository $drinkTypeRepository,
    ) {}

    /**
     * Get all orders
     *
     * @return Order[]
     */
    public function getAllOrders(): array
    {
        return $this->orderRepository->findAll();
    }

    /**
     * Get an order by ID
     *
     * @param int $id
     * @return Order|null
     */
    public function getOrderById(int $id): null|Order
    {
        return $this->orderRepository->find($id);
    }

    /**
     * Get orders by status
     *
     * @return Order[]
     */
    public function getOrdersByStatus(OrderStatus $status): array
    {
        return $this->orderRepository->findByStatus($status);
    }

    public function getActiveOrders(): array
    {
        return $this->orderRepository->findByMultipleStatus([
            OrderStatus::NEW,
            OrderStatus::ORDERED,
            OrderStatus::IN_WORK,
        ]);
    }

    /**
     * Get the most recent order in "new" or "in work" status
     *
     * @return Order|null
     */
    public function getMostRecentActiveOrder(): null|Order
    {
        $newOrders = $this->orderRepository->findByStatus(OrderStatus::NEW);
        $inWorkOrders = $this->orderRepository->findByStatus(OrderStatus::IN_WORK);

        $activeOrders = array_merge($newOrders, $inWorkOrders);

        // Sort by creation date (newest first)
        usort($activeOrders, fn(Order $a, Order $b): int => $b->getCreatedAt() <=> $a->getCreatedAt());

        return $activeOrders[0] ?? null;
    }

    /**
     * Check if there are any orders in "new" or "in work" status
     *
     * @return bool
     */
    public function hasActiveOrders(): bool
    {
        $newOrders = $this->orderRepository->findByStatus(OrderStatus::NEW);
        $inWorkOrders = $this->orderRepository->findByStatus(OrderStatus::IN_WORK);

        return $newOrders !== [] || $inWorkOrders !== [];
    }

    /**
     * Get orders within a specific date range
     *
     * @param DateTimeImmutable $start
     * @param DateTimeImmutable $end
     * @return Order[]
     */
    public function getOrdersByDateRange(DateTimeImmutable $start, DateTimeImmutable $end): array
    {
        return $this->orderRepository->findByDateRange($start, $end);
    }

    /**
     * Create a new order
     *
     * @param array<int, array{drinkTypeId: int, quantity: int}> $items
     * @return Order
     * @throws InvalidArgumentException If a drink type ID is invalid
     */
    public function createOrder(array $items): Order
    {
        $order = new Order();
        $this->orderRepository->save($order);

        foreach ($items as $item) {
            $drinkType = $this->drinkTypeRepository->find($item['drinkTypeId']);
            if ($drinkType === null) {
                throw new InvalidArgumentException("Invalid drink type ID: {$item['drinkTypeId']}");
            }

            $orderItem = new OrderItem();
            $orderItem->setDrinkType($drinkType);
            $orderItem->setQuantity($item['quantity']);
            $orderItem->setOrder($order);
            $this->orderItemRepository->save($orderItem);
        }

        return $order;
    }
}
